/******************************************************************
*
*	CyberLink for C++
*
*	Copyright (C) Satoshi Konno 2002-2003
*
*	File: ClockMain.cpp
*
*	Revision;
*
*	09/23/03
*		- first revision
*
******************************************************************/

#include <Carbon/Carbon.h>
#include <iostream>

#include "ClockDevice.h"

using namespace std;
using namespace CyberLink;

ClockDevice *clockDev;

////////////////////////////////////////////////////////// 
//  InitApp
////////////////////////////////////////////////////////// 

void InitApp()
{
	clockDev = new ClockDevice();
	clockDev->start();
}

void ExitApp()
{
	clockDev->stop();
	delete clockDev;
}

pascal void ClockUpdate(EventLoopTimerRef timerRef, void *userData)
{
	clockDev->update();
	WindowRef window = (WindowRef)userData;
	Rect clockRect;
	SetRect(&clockRect, 0,0,320,240);
	InvalWindowRect(window, &clockRect);
}

/////////////////////////////////////////////////////////////////////////////////
// DrawClock
/////////////////////////////////////////////////////////////////////////////////

pascal OSStatus DrawClock(EventHandlerCallRef myHandler, EventRef event, void* userData)
{
	Clock clock;
	std::string dateStr;
	std::string timeStr;
	clock.getDateString(dateStr);
	clock.getTimeString(timeStr);
		
	SetPort(GetWindowPort((WindowRef)userData));
	
	PicHandle clockPict = GetPicture(29876);
	//PicPtr clockPictPtr = *clockPict;
	//Rect clockRect = clockPictPtr->picFrame;
	Rect clockRect;
	SetRect(&clockRect, 0,0,320,240);
	DrawPicture(clockPict, &clockRect);
	
	short int fontNum;
	GetFNum("\pCourier", &fontNum);
	TextFont(fontNum);
	TextFace(bold);

	RGBColor col = {0, 0, 0};
	RGBForeColor(&col);
	
	//// Date String ////
	
	TextSize(20);
	MoveTo(60, 70);
	Str255 pDateStr;
	CopyCStringToPascal(dateStr.c_str(), pDateStr);
	DrawString(pDateStr);

	//// Time String ////

	int timeStrX = 65;
	int timeStrY = 140;
	int timeStrWidth = 180;
	int timeStrOffset = 20;
	TextSize(60);
	MoveTo(timeStrX, timeStrY);
	Str255 pTimeStr;
	CopyCStringToPascal(timeStr.c_str(), pTimeStr);
	DrawString(pTimeStr);
	
	//// Second Bar ////
	
	int DEFAULT_SECOND_BLOCK_HEIGHT = 8;
	int sec = clock.getSecond();
	int secBarBlockSize = timeStrWidth / 60;
	int secBarBlockY = timeStrY + timeStrOffset;
	for (int n=0; n<sec; n++) {
		int x = timeStrX + (secBarBlockSize*n);
		Rect rect;
		SetRect(&rect, x, secBarBlockY, x + (secBarBlockSize - 1), secBarBlockY + DEFAULT_SECOND_BLOCK_HEIGHT);
		PaintRect(&rect);
	}

	if (sec != 0 && (sec % 10) == 0) {
		int x = timeStrX + (secBarBlockSize*sec);
		TextSize(10);
		MoveTo(x + secBarBlockSize, secBarBlockY + DEFAULT_SECOND_BLOCK_HEIGHT);
		char secStr[8];
		sprintf(secStr, "%02d", sec);
		Str255 pSecStr;
		CopyCStringToPascal(secStr, pSecStr);
		DrawString(pSecStr);
	}
	
	return noErr;
}

/////////////////////////////////////////////////////////////////////////////////
// main
/////////////////////////////////////////////////////////////////////////////////

int main(int argc, char* argv[])
{
    IBNibRef 		nibRef;
    WindowRef 		window;
	
	EventTypeSpec	drawEventType = {kEventClassWindow, kEventWindowDrawContent}; 

    OSStatus		err;
	
    // Create a Nib reference passing the name of the nib file (without the .nib extension)
    // CreateNibReference only searches into the application bundle.
    err = CreateNibReference(CFSTR("clock"), &nibRef);
    require_noerr( err, CantGetNibRef );
    
    // Once the nib reference is created, set the menu bar. "MainMenu" is the name of the menu bar
    // object. This name is set in InterfaceBuilder when the nib is created.
    err = SetMenuBarFromNib(nibRef, CFSTR("MenuBar"));
    require_noerr( err, CantSetMenuBar );
  
    // Then create a window. "MainWindow" is the name of the window object. This name is set in 
    // InterfaceBuilder when the nib is created.
    err = CreateWindowFromNib(nibRef, CFSTR("MainWindow"), &window);
    require_noerr( err, CantCreateWindow );

    // We don't need the nib reference anymore.
    DisposeNibReference(nibRef);

	InstallWindowEventHandler( window, NewEventHandlerUPP(DrawClock), 1, &drawEventType, (void *)window, NULL );

	EventLoopTimerRef tref;
	InstallEventLoopTimer(GetCurrentEventLoop(), 0, 1, NewEventLoopTimerUPP(ClockUpdate), (void *)window, &tref);
	
	InitApp();
	
    // The window was created hidden so show it.
    ShowWindow( window );
  
	/*  
	char cdir[256];
	getcwd(cdir, 255);
	cout << cdir << endl;
	*/
	
    // Call the event loop
    RunApplicationEventLoop();

	ExitApp();
	
	CantCreateWindow:
	CantSetMenuBar:
	CantGetNibRef:
	
	return err;
}

